package quake.quake_tensorflow_face_rec;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Parcelable;
import android.provider.Settings;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import org.json.JSONObject;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import quake.quake_tensorflow_face_rec.detection.DetectorActivity;
import quake.quake_tensorflow_face_rec.utils.Constants;
import quake.quake_tensorflow_face_rec.utils.JSONUtils.JsonUtils;

/**
 * QuakeTensorflowFaceRecPlugin
 */
public class QuakeTensorflowFaceRecPlugin extends FlutterActivity implements FlutterPlugin, MethodCallHandler, ActivityAware {
    private static final String TAG = "QuakeTensorflow";
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    private Context mContext;
    private Activity mActivity;
    Result mResult;
    private String signInOrOut;

    private String workerFaceData;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "quake_tensorflow_face_rec");
        channel.setMethodCallHandler(this);
        mContext = flutterPluginBinding.getApplicationContext();
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        mResult = result;
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else if (call.method.equals("jumpToFaceRec")) {
            signInOrOut = call.argument("signInOrOut");
            workerFaceData = call.argument("workerFaceData");
            Log.d(TAG, "onMethodCall: 获取参数tfFaceFeature="+workerFaceData);
            checkPermission();

        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        mActivity = binding.getActivity();
        binding.addActivityResultListener(new PluginRegistry.ActivityResultListener() {
            @Override
            public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
                if (requestCode == Constants.FACE_REC_RESULT) {
                    Log.d(TAG, "onActivityResult: 到了");
                    if (data != null) {
                        String log = data.getStringExtra("test");
                        String signInOrOut = data.getStringExtra("signInOrOut");
                        String worker_id = data.getStringExtra("id");
                        if (worker_id == null) {
                            worker_id = "";
                        }
                        String faceFeature = data.getStringExtra("faceFeature");
                        if (faceFeature == null) {
                            faceFeature = "";
                        }
                        String facePicUrl = data.getStringExtra("facePicUrl");
                        if (facePicUrl == null) {
                            facePicUrl = "";
                        }
                        Log.d(TAG, "onActivityResult: worker_id=" + worker_id);
                        Log.d(TAG, "onActivityResult: signInOrOut=" + signInOrOut);
                        Log.d(TAG, "onActivityResult: log=" + log);
                        Log.d(TAG, "onActivityResult: faceFeature=" + faceFeature);
                        Log.d(TAG, "onActivityResult: facePicUrl=" + facePicUrl);
                        JSONObject jsonObject = new JSONObject();
                        jsonObject = JsonUtils.inputValue2JSONObject(jsonObject, "worker_id", worker_id,
                                "signInOrOut", signInOrOut, "facePicUrl", facePicUrl,
                                "log", log, "faceFeature", faceFeature);
                        mResult.success(jsonObject.toString());
                    }
                }
                return false;
            }
        });
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {

    }


    private static final int REQUEST_CODE_WRITE_SETTINGS = 1;
    List<String> mPermissionList = new ArrayList<>();
    private static final int PERMISSION_REQUEST = 1;
    String[] permissions = new String[]{
            Manifest.permission.CAMERA,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.READ_EXTERNAL_STORAGE,
    };

    //检查权限并授权
    @RequiresApi(api = Build.VERSION_CODES.M)
    private void checkPermission() {
//        requestWriteSettings();
        mPermissionList.clear();

        //判断哪些权限未授予
        for (int i = 0; i < permissions.length; i++) {
            if (ContextCompat.checkSelfPermission(mContext, permissions[i]) != PackageManager.PERMISSION_GRANTED) {
                mPermissionList.add(permissions[i]);
            }
        }
        /**
         * 判断是否为空
         */
        if (mPermissionList.isEmpty()) {//未授予的权限为空，表示都授予了
            Toast.makeText(mContext, "跳转到tensorFlow", Toast.LENGTH_SHORT).show();
            Intent intent = new Intent(mActivity, DetectorActivity.class);
            intent.putExtra("signInOrOut", signInOrOut);
            intent.putExtra("workerFaceData", workerFaceData);
            mActivity.startActivityForResult(intent, Constants.FACE_REC_RESULT);
        } else {//请求权限方法
            String[] permissions = mPermissionList.toArray(new String[mPermissionList.size()]);//将List转为数组
            for (int i = 0; i < permissions.length; i++) {
                Log.d(TAG, "checkPermission: 遍历未授权权限" + permissions[i]);
            }
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                // 选择你需要申请的权限
                for (int i = 0; i < permissions.length; i++) {
                    int state = ContextCompat.checkSelfPermission(mContext, permissions[i]);
                    if (state != PackageManager.PERMISSION_GRANTED) { // 判断权限的状态
                        ActivityCompat.requestPermissions(mActivity, permissions, PERMISSION_REQUEST); // 申请权限
                        return;
                    }
                }
            }
        }


    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && requestCode == PERMISSION_REQUEST) {
            for (int i = 0; i < permissions.length; i++) {
                Log.d(TAG, "onRequestPermissionsResult: " + permissions[i] + "是否授权：" + (grantResults[i] == PackageManager.PERMISSION_GRANTED));
                if (grantResults[i] != PackageManager.PERMISSION_GRANTED) { // 用户点的拒绝，仍未拥有权限
//                    Log.d("alertDialog","alertDialog:"+(alertDialog!=null)+":"+alertDialog.isShowing());
                    if (alertDialog == null || !alertDialog.isShowing()) {
                        Toast.makeText(mContext, "请在设置中打开存储权限", Toast.LENGTH_SHORT).show();
                        showWarningDialog();
                    }
                    return;
                } else {
                    if (alertDialog != null) {
                        alertDialog.dismiss();

                    }
                }
            }
        }

    }

    private AlertDialog alertDialog;

    //权限弹窗
    @RequiresApi(api = Build.VERSION_CODES.M)
    private void showWarningDialog() {
        Log.d(TAG, "showWarningDialog: 系统设置" + Settings.System.canWrite(mContext));
        alertDialog = new AlertDialog.Builder(mContext)
                .setTitle("警告！")
                .setMessage("请前往设置->应用->夸克微人脸系统->权限中打开 读写存储、定位  权限，否则功能无法正常运行！")
                .setPositiveButton("前往设置", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        // 可以选择添加如下代码在系统设置中打开该应用的设置页面
                        Intent intent = new Intent();
                        intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                        Uri uri = Uri.fromParts("package", mContext.getPackageName(), null);
                        intent.setData(uri);
                        mContext.startActivity(intent);
                    }
                }).setNegativeButton("拒绝", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        // 一般情况下如果用户不授权的话，功能是无法运行的，做退出处理
                        mActivity.finish();
                    }
                }).setCancelable(false).show();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_CODE_WRITE_SETTINGS) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                //Settings.System.canWrite方法检测授权结果
                if (Settings.System.canWrite(mContext.getApplicationContext())) {
                    Toast.makeText(mContext, "获取了权限", Toast.LENGTH_SHORT).show();
                } else {
                    Toast.makeText(mContext, "您拒绝了权限", Toast.LENGTH_SHORT).show();
                }
            }
        }


    }
}
