/**
 * Copyright (C), 2021-2021, 夸克科技有限公司
 * FileName: BasePresenter
 * Author: Administrator
 * Date: 2021/10/17 11:52
 * Description: 公用的Presenter业务处理
 * History:
 * <author> <time> <version> <desc>
 * 作者姓名 修改时间 版本号 描述
 */
package quake.quake_tensorflow_face_rec.baseMvp;

import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleObserver;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.OnLifecycleEvent;

import java.lang.ref.WeakReference;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

/**
 * @ClassName: BasePresenter
 * @Description: 公用的Presenter业务处理，实现了声明周期的管理
 * @Author: Administrator
 * @Date: 2021/10/17 11:52
 */
public class BasePresenter<V extends IBaseView, M extends IBaseModel> implements IBasePresenter<V>, LifecycleObserver {
    private static final String TAG = "BasePresenter";
    protected WeakReference<V> mView;
    protected M mModel;

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public BasePresenter(){
        //通过反射新建model
        Type[] types = ((ParameterizedType)this.getClass().getGenericSuperclass()).getActualTypeArguments();
        Log.d(TAG, "BasePresenter: type:"+types[0]);
        try {
            mModel = (M)((Class)types[1]).newInstance();
        } catch (IllegalAccessException | InstantiationException e) {
            e.printStackTrace();
            Log.d(TAG, "BasePresenter: 错误日志："+e.getMessage());
        }
    }

    @Override
    public void attach(V view) {
        this.mView = new WeakReference(view);
        //关联生命周期
        mView.get().getLifecycle().addObserver(this);
    }

    @Override
    public void detach() {
        if (mView != null) {
            mView.get().getLifecycle().removeObserver(this);
            mView = null;
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    public void onDestroy(LifecycleOwner owner) {
        detach();
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_PAUSE)
    public void onPause(LifecycleOwner owner) {
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_RESUME)
    public void onResume(LifecycleOwner owner) {
    }
}
