/*
 * Copyright 2019 The TensorFlow Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package quake.quake_tensorflow_face_rec.detection;

import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.RectF;
import android.hardware.camera2.CameraCharacteristics;
import android.media.ImageReader.OnImageAvailableListener;
import android.os.Build;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;
import android.util.Size;
import android.view.View;
import android.view.WindowManager;
import android.widget.CompoundButton;
import android.widget.ImageView;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.fragment.app.FragmentTransaction;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.mlkit.vision.common.InputImage;
import com.google.mlkit.vision.face.Face;
import com.google.mlkit.vision.face.FaceDetector;


import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import quake.quake_tensorflow_face_rec.R;
import quake.quake_tensorflow_face_rec.baseMvp.annotation.InjectPresenter;
import quake.quake_tensorflow_face_rec.databinding.TfeOdActivityCameraBinding;
import quake.quake_tensorflow_face_rec.detection.env.ImageUtils;
import quake.quake_tensorflow_face_rec.detection.env.Logger;
import quake.quake_tensorflow_face_rec.detection.javaBean.FaceCount;
import quake.quake_tensorflow_face_rec.detection.javaBean.FaceResults;
import quake.quake_tensorflow_face_rec.detection.mvp.contract.IRecognizeContract;
import quake.quake_tensorflow_face_rec.detection.mvp.presenter.RecognizePresenter;
import quake.quake_tensorflow_face_rec.detection.tflite.SimilarityClassifier;
import quake.quake_tensorflow_face_rec.utils.BitmapUtils;
import quake.quake_tensorflow_face_rec.utils.BundleUtils;
import quake.quake_tensorflow_face_rec.utils.Constants;
import quake.quake_tensorflow_face_rec.utils.FileUtils;
import quake.quake_tensorflow_face_rec.utils.JSONUtils.JsonUtils;
import quake.quake_tensorflow_face_rec.utils.MyDatabaseHelper;
import quake.quake_tensorflow_face_rec.utils.RunningRing;
import quake.quake_tensorflow_face_rec.utils.TensorFlowUtils.FaceDetectorHelper;


/**
 * An activity that uses a TensorFlowMultiBoxDetector and ObjectTracker to detect and then track
 * objects.
 */
public class DetectorActivity extends CameraActivity implements OnImageAvailableListener, View.OnClickListener,
        IRecognizeContract.View {
    private static final Logger LOGGER = new Logger();
    private static final String TAG = "DetectorActivity";
    private Boolean isCentered = false;

    private int count = 0;
    private HashMap<String, Integer> faceMap;

    String label = "陌生人";
    float confidence = -1f;
    Integer color = Color.BLUE;
    Object extra = null;
    float[][] embeddings = null;

    // FaceNet
//  private static final int TF_OD_API_INPUT_SIZE = 160;
//  private static final boolean TF_OD_API_IS_QUANTIZED = false;
//  private static final String TF_OD_API_MODEL_FILE = "facenet.tflite";
//  //private static final String TF_OD_API_MODEL_FILE = "facenet_hiroki.tflite";

    // MobileFaceNet
    private static final int TF_OD_API_INPUT_SIZE = 112;
    private static final boolean TF_OD_API_IS_QUANTIZED = false;
    private static final String TF_OD_API_MODEL_FILE = "mobile_face_net.tflite";


    private static final String TF_OD_API_LABELS_FILE = "file:///android_asset/labelmap.txt";

    private static final DetectorMode MODE = DetectorMode.TF_OD_API;
    // Minimum detection confidence to track a detection.
    private static final float MINIMUM_CONFIDENCE_TF_OD_API = 0.5f;
    private static final boolean MAINTAIN_ASPECT = false;

    //    private static final Size DESIRED_PREVIEW_SIZE = new Size(640, 480);
    private int desired_width, desired_height;
//    private Size DESIRED_PREVIEW_SIZE = new Size(480, 360);
//    private Size DESIRED_PREVIEW_SIZE = new Size(1920, 1080);
    //private static final int CROP_SIZE = 320;
    //private static final Size CROP_SIZE = new Size(320, 320);


    private static final boolean SAVE_PREVIEW_BITMAP = false;
    private static final float TEXT_SIZE_DIP = 10;
    private Integer sensorOrientation;

    private SimilarityClassifier detector;

    //    private long lastProcessingTimeMs;
    private Bitmap rgbFrameBitmap = null;
    private Bitmap croppedBitmap = null;
    private Bitmap cropCopyBitmap = null;

    private boolean computingDetection = false;
    private boolean addPending = false;
    //private boolean adding = false;

    private long timestamp = 0;

    private Matrix frameToCropTransform;
    private Matrix cropToFrameTransform;
    //private Matrix cropToPortraitTransform;


    // Face detector
    private FaceDetector faceDetector;

    // here the preview image is drawn in portrait way
    private Bitmap portraitBmp = null;
    // here the face is cropped and drawn
    private Bitmap faceBmp = null;

    private FloatingActionButton fabAdd;

    //private HashMap<String, Classifier.Recognition> knownFaces = new HashMap<>();

    private String phone;

    private List<List<Face>> faceList = new ArrayList<>();
    private String hint;

    private BundleUtils bundleUtils;
    private FragmentTransaction ft;
    private Boolean flag = false;
    private long secondTime;
    private long firstTime;
    private boolean recFlag;
    private int unSureCount = 0;
    private int recognizeCount = 0;
    private int extraFlag;
    private boolean isDetected = true;
    private ArrayList<FaceResults> listFaceResults = new ArrayList<>();


    private RunningRing runningRing;
    private SQLiteDatabase db;
    private MyDatabaseHelper helper = new MyDatabaseHelper(this);
    private String signInOrOut;
    private String mLog;
    private JSONArray workerFaceData;

    private TfeOdActivityCameraBinding binding;
    @InjectPresenter
    RecognizePresenter presenter;

    public DetectorActivity() {
    }

    @RequiresApi(api = Build.VERSION_CODES.N)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        db = helper.getWritableDatabase();
        initViews();
        signInOrOut();
        //FaceDetector用于在提供的图像中查找人脸的人脸检测客户端
        faceDetector = FaceDetectorHelper.getInstance().createFaceDetector();
        Log.d(TAG, "onCreate: faceDetector=" + faceDetector);
    }

    private void signInOrOut() {
        Intent intent = getIntent();
        JSONObject str = null;
        try {
            str = new JSONObject(intent.getStringExtra("workerFaceData"));
            workerFaceData =JsonUtils.string2JSONArray(str.getString("workerFaceData"));
        } catch (JSONException e) {
            e.printStackTrace();
        }
        Log.d(TAG, "signInOrOut: 人脸数据="+workerFaceData);
        Log.d(TAG, "signInOrOut: str="+str);

        signInOrOut = intent.getStringExtra("signInOrOut");
        Log.d(TAG, "signInOrOut: 收到的Intent=" + signInOrOut);
    }

    private void getFaceInfo(SimilarityClassifier detector) {
        Log.d(TAG, "getFaceInfo: workerFaceData="+workerFaceData);
        presenter.registerFace(workerFaceData, detector);
    }

    @Override
    public void onBackPressed() {
        //super.onBackPressed();
        this.finish();
    }
    //相机预览图片宽高确定后，回调onPreviewSizeChosen
    //onPreviewSizeChosen会调用到ClassifierActivity中。它主要做了两件事，构造分类器classifier,它是模型分类预测的一个比较关键的类。
    //另外就是预处理输入图片，如裁剪到和模型训练所使用的图片相同的尺寸

    //图片预览展现出来时回调，主要是构造分类器classifier，和裁剪输入图片为224*224
    @Override
    public void onPreviewSizeChosen(final Size size, final int rotation) {
//构造检测器，利用TensorFlow训练出来的Model，也就是.tflite文件。这是后面人脸检测的关键
        detector = presenter.createClassifier(this, this);
//        cacheFace(detector);
        getFaceInfo(detector);
        previewWidth = size.getWidth();
        previewHeight = size.getHeight();
        sensorOrientation = rotation - getScreenOrientation();
        LOGGER.i("Camera orientation relative to screen canvas: %d", sensorOrientation);
        LOGGER.i("Initializing at size %dx%d", previewWidth, previewHeight);
        rgbFrameBitmap = Bitmap.createBitmap(previewWidth, previewHeight, Config.ARGB_8888);
        int targetW, targetH;
        if (sensorOrientation == 90 || sensorOrientation == 270) {
            targetH = previewWidth;
            targetW = previewHeight;
        } else {
            targetW = previewWidth;
            targetH = previewHeight;
        }
        int cropW = (int) (targetW / 2.0);
        int cropH = (int) (targetH / 2.0);
        croppedBitmap = Bitmap.createBitmap(cropW, cropH, Config.ARGB_8888);
        portraitBmp = Bitmap.createBitmap(targetW, targetH, Config.ARGB_8888);
        faceBmp = Bitmap.createBitmap(TF_OD_API_INPUT_SIZE, TF_OD_API_INPUT_SIZE, Config.ARGB_8888);
        //将照相机获取的原始图片，转换为224*224的图片，用来作为模型预测的输入
        frameToCropTransform =
                ImageUtils.getTransformationMatrix(
                        previewWidth, previewHeight,
                        cropW, cropH,
                        sensorOrientation, MAINTAIN_ASPECT);

        cropToFrameTransform = new Matrix();
        frameToCropTransform.invert(cropToFrameTransform);

    }

//    private void detect() {
//        BitmapFactory.Options options = new BitmapFactory.Options();
//        options.inPreferredConfig = Bitmap.Config.RGB_565;
//        Bitmap liu = BitmapFactory.decodeResource(getResources(), R.drawable.liu1, options);
//        InputImage image = InputImage.fromBitmap(liu, 0);
//        faceDetector.process(image).addOnSuccessListener(new OnSuccessListener<List<Face>>() {
//            @Override
//            public void onSuccess(List<Face> faces) {
//
//                onFacesDetected(0, faces, false);
//                List<SimilarityClassifier.Recognition> mappedRecognitions = process5face();
//                updateResults(0, mappedRecognitions);
////                List<SimilarityClassifier.Recognition> mappedRecognitions =detector.recognizeImage(liu,"recognize");
//                for (int i = 0; i < mappedRecognitions.size(); i++) {
//                    Log.d("mappedRecognitions", "liumappedRecognitions:" + mappedRecognitions.get(i).getDistance());
//                }
////                updateResults(0, mappedRecognitions);
//            }
//        });
//    }

    //回调的第一个入口
    //处理图像，包括人脸检测
    //processImage()先做图片绘制方面的工作，将相机捕获的图片绘制出来。然后利用分类器classifier来检测图片
    @Override
    protected void processImage() {
        ++timestamp;
        final long currTimestamp = timestamp;
//        trackingOverlay.postInvalidate();

        // No mutex needed as this method is not reentrant.
        //由于此方法不可重入，因此不需要互斥锁
        if (computingDetection) {
            readyForNextImage();
            return;
        }
        computingDetection = true;

        LOGGER.i("Preparing image " + currTimestamp + " for detection in bg thread.");
        //图片的绘制等
        rgbFrameBitmap.setPixels(getRgbBytes(), 0, previewWidth, 0, 0, previewWidth, previewHeight);
        readyForNextImage();
        final Canvas canvas = new Canvas(croppedBitmap);
        canvas.drawBitmap(rgbFrameBitmap, frameToCropTransform, null);
        // For examining the actual TF input.
        if (SAVE_PREVIEW_BITMAP) {
            ImageUtils.saveBitmap(croppedBitmap);
        }
        //要检测图像中的人脸，请从 Bitmap、media.Image、ByteBuffer、字节数组或设备上的文件创建 InputImage 对象。
        // 然后，将 InputImage 对象传递给 FaceDetector 的 process 方法。
        //通过bitmap对象创建InputImage
        InputImage image = InputImage.fromBitmap(croppedBitmap, 0);
        //检测提供的InputImage中的人脸
        //如果没有检测到人脸，返回的任务将包含一个空列表

/*        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inScaled = false;
        options.inPreferredConfig = Bitmap.Config.RGB_565;
//        options.inPreferredConfig = Config.ARGB_8888;
        Bitmap liu = BitmapFactory.decodeResource(getResources(),R.drawable.liu2,options);
        Log.d("bitmap","liu.getWidth()"+liu.getWidth()+",liu.getHeight()"+liu.getHeight());

//        InputImage kkk = InputImage.fromBitmap(liu, 0);
//        int wi = DataConvertion.dip2px(this,liu.getWidth());
//        int he = DataConvertion.dip2px(this,liu.getHeight());
        int width = liu.getWidth();
        int height = liu.getHeight();
        float  scaleWidth = (float) 112/width;
        float scaleHeight = (float) 112/height;
        Matrix matrix = new Matrix();
        matrix.postScale(scaleWidth,scaleHeight);
        Log.d("bitmap","liu.getWidth()"+liu.getWidth()+",liu.getHeight()"+liu.getHeight());
        Bitmap newLiu = Bitmap.createBitmap(liu,0,0,width,height,matrix,false);

        Log.d("bitmap","liu.getWidth()"+newLiu.getWidth()+",liu.getHeight()"+newLiu.getHeight());
        detector.recognizeImage(liu,"register");*/
        Log.d(TAG, "processImage: isDetected = " + isDetected);
        if (isDetected) {
            faceDetector
                    .process(image).addOnFailureListener(new OnFailureListener() {
                @Override
                public void onFailure(@NonNull Exception e) {
                    Log.d(TAG, "onFailure: 错误日志=" + e.getMessage());
                }
            }).addOnSuccessListener(new OnSuccessListener<List<Face>>() {
                @Override
                public void onSuccess(List<Face> faces) {
                    Log.d("onSuccess", "faces.size()" + faces.size());
                    isCentered = false;
                    if (listFaceResults.size() > 0) {
                        runningRing.setVisibility(View.VISIBLE);
                        runningRing.controlSpeed(listFaceResults.size());
                    } else {
                        runningRing.setVisibility(View.GONE);
                    }
                    Log.d("tag", "当前时间差" + (System.currentTimeMillis() - firstTime));
                    Log.d("tag", "faces是否为空" + faces.equals(null));
                    Log.d("tag", "当前时间差firstTime" + firstTime);
                    Log.d("tag", "当前时间差secondTime" + secondTime);
                    Log.d("listFaceResults", "!recFlag " + !recFlag + "!isCentered" + !isCentered);
                    if (secondTime - firstTime > 500 || (!recFlag && !isCentered && faces.size() == 0)) {
                        listFaceResults.clear();
                        count = 0;
//                            flag = false;
                        Log.d("tag", "flag2:" + flag);
                    }
                    //利用分类器classifier对图片进行预测分析，得到图片为人脸的概率，比较耗时，放在子线程中
                    runInBackground(
                            new Runnable() {
                                @Override
                                public void run() {
                                    Log.d("tag", "没检测到人" + faces.equals(null));
                                    if (faces != null) {
                                        //进行人脸检测
                                        //addPending是是否进注册的标识
                                        Log.d("tag", "run检测结果" + faces);
                                        onFacesDetected(faces);
//                                                if(listFaceResults.size() == 20){
                                        Log.d("recognizeResult", "listFaceResults.size()" + listFaceResults.size());

                                        List<SimilarityClassifier.Recognition> mappedRecognitions = process5face();
                                        updateResults(mappedRecognitions);
                                        recFlag = false;
                                    }
                                }
                            });
                }
            });
        }

    }


    @Override
    protected int getLayoutId() {
        return R.layout.tfe_od_camera_connection_fragment_tracking;
    }

    @Override
    protected Size getDesiredPreviewFrameSize() {
        WindowManager wm = (WindowManager) this
                .getSystemService(Context.WINDOW_SERVICE);
        int width = wm.getDefaultDisplay().getWidth();
        int height = wm.getDefaultDisplay().getHeight();
        Log.d("getDefaultDisplay", "宽高比width:" + width + ",height:" + height + "宽高比：" + (float) height / width);
        Size DESIRED_PREVIEW_SIZE = new Size(height, width);
        return DESIRED_PREVIEW_SIZE;
//        return size;
    }

    @Override
    public void onClick(View v) {
        if (v == binding.backward) {
//                timer.onFinish();
            finish();
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

    }

    @Override
    public void showErrorMessage(String error) {

    }

    @Override
    public void createFunctions() {

    }

    @Override
    protected RecognizePresenter createPresenter() {
        return new RecognizePresenter();
    }

    @Override
    public void initViews() {
        binding = TfeOdActivityCameraBinding.inflate(getLayoutInflater());
        runningRing = binding.ring;
        View rootView = binding.getRoot();
        setContentView(rootView);
        ImageView backward = binding.backward;
        backward.setOnClickListener(this);
    }


    // Which detection model to use: by default uses Tensorflow Object Detection API frozen
    // checkpoints.
    //使用哪种检测模型：默认情况下使用Tensorflow对象检测API冻结检查点
    private enum DetectorMode {
        TF_OD_API;
    }

    @Override
    protected void setUseNNAPI(final boolean isChecked) {
        runInBackground(() -> detector.setUseNNAPI(isChecked));
    }

    @Override
    protected void setNumThreads(final int numThreads) {
        runInBackground(() -> detector.setNumThreads(numThreads));
    }

    // Face Processing
    //预处理预览图片，裁剪、旋转等操作
    //srcWidth,srcHeight为预览图片宽高，dstWidth dstHeight为训练模型时使用的图片的宽高
    //applyRotation旋转角度，必须是90的倍数
    private Matrix createTransform(
            final int srcWidth,
            final int srcHeight,
            final int dstWidth,
            final int dstHeight,
            final int applyRotation) {
        //定义预处理后的图片像素矩阵
        Matrix matrix = new Matrix();
        //处理旋转
        if (applyRotation != 0) {
            //旋转只能处理90度的倍数
            if (applyRotation % 90 != 0) {
                LOGGER.w("Rotation of %d % 90 != 0", applyRotation);
            }
            // Translate so center of image is at origin.
            //translate平移，保持圆心不变
            matrix.postTranslate(-srcWidth / 2.0f, -srcHeight / 2.0f);

            // Rotate around origin.
            //rotate旋转
            matrix.postRotate(applyRotation);
        }
//        // Account for the already applied rotation, if any, and then determine how
//        // much scaling is needed for each axis.
//        final boolean transpose = (Math.abs(applyRotation) + 90) % 180 == 0;
//        final int inWidth = transpose ? srcHeight : srcWidth;
//        final int inHeight = transpose ? srcWidth : srcHeight;
        if (applyRotation != 0) {
            //平移变换
            // Translate back from origin centered reference to destination frame.
            matrix.postTranslate(dstWidth / 2.0f, dstHeight / 2.0f);
        }
        return matrix;
    }

    @Override
    public synchronized void onDestroy() {
        super.onDestroy();
        if (croppedBitmap != null && !croppedBitmap.isRecycled()) {
            //回收并且置为null
            croppedBitmap.recycle();
            croppedBitmap = null;
        } else if (faceBmp != null && !faceBmp.isRecycled()) {
            faceBmp.recycle();
            faceBmp = null;
        } else if (portraitBmp != null && !portraitBmp.isRecycled()) {
            portraitBmp.recycle();
            portraitBmp = null;
        } else if (rgbFrameBitmap != null && !rgbFrameBitmap.isRecycled()) {
            rgbFrameBitmap.recycle();
            rgbFrameBitmap = null;
        }
        if (db != null) {
            db.close();
        }
        System.gc();
    }

    //存储打卡图像到外部存储
    private String saveSignPicToLocal(String uuid) {
        String file = FileUtils.getDiskCacheDir(this);
        File appCacheDir = new File(file, uuid);

        Log.d("saveSignPicToLocal", "file路径:" + file);
        new Thread(new Runnable() {
            @Override
            public void run() {
                //存头像
                BitmapUtils.saveBitmap(appCacheDir.getPath() + ".jpeg", portraitBmp);
                Log.d("saveSignPicToLocal", "本地存储图像路径:" + appCacheDir.getPath() + ".jpeg");
            }
        }).start();
        return appCacheDir.getPath() + ".jpeg";
    }

    //即时更行结果
    private void updateResults(final List<SimilarityClassifier.Recognition> mappedRecognitions) {
        for (int i = 0; i < mappedRecognitions.size(); i++) {
            Log.d("mappedRecognitions", "mappedRecognitions:" + mappedRecognitions.get(i).getDistance());
        }
        computingDetection = false;
        //adding = false;
        Log.d("updateResults", "isCentered:" + isCentered);
        Log.d("4mappedRecognitions", "mappedRecognitions.size():" + mappedRecognitions.size());
        if (mappedRecognitions.size() > 0) {
            Log.d("updateResults", "更新结果");
            SimilarityClassifier.Recognition rec;
            LOGGER.i("Adding results");
            Log.d("recognizeResult", "开始遍历");
            FaceCount faceCount = new FaceCount();
            for (int i = 0; i < mappedRecognitions.size(); i++) {
                rec = mappedRecognitions.get(i);
                Log.d("updateResults", "rec.getExtra():" + rec.getExtra());
                Log.d("what", "进来了吗？" + Arrays.deepToString(rec.getEmbeddings()));
                if (rec.getExtra() != null) {
                    count++;
//                        Log.d("recognizeResult", "count？" + count);
                    Log.d("title", "rec.getTitle()为空吗？" + rec.getTitle().equals(""));
//                        if (!rec.getTitle().equals("")) {
                    //检测到的id计数加1
                    faceMap = faceCount.addFaceCount(rec.getTitle());//存的是  id:匹配到的次数   这个函数每调用一次 次数就增加一次
//                        faceMap.add(faceCount);
                    Log.d("count", "faceMap？" + faceMap);
                    for (Map.Entry<String, Integer> entry : faceMap.entrySet()) {
                        Log.d("count", "entry.getValue()？" + entry.getValue());
                        Log.d("entry_key_value", "key:" + entry.getKey() + ",value:" + entry.getValue());
                        Log.d("recognizeResult", "faceMap.size:" + faceMap.entrySet().size() + ",getDistance:" + rec.getDistance());
                        Log.d("recognizeResult", entry.getKey() + "|" + rec.getTitle() + ",结果为" + count + "帧里" + entry.getKey() + "匹配到" + entry.getValue()
                                + "帧");
//                            Toast.makeText(this, count + "帧里有" + entry.getValue() + "帧是" + entry.getKey(), Toast.LENGTH_SHORT).show();
                        Log.e("recognizeResult", "tostring" + rec.toString());
                        if (count == 5) {
                            Log.d("recognizeResult", "遍历结束");
                            flag = true;
                            int count2 = 0;
                            Log.e("recognizeResult", (entry.getValue() >= 3) + entry.getKey() + "有" + entry.getValue() + "帧匹配"
                                    + rec.getTitle());
                            Log.d("recognizeResult", entry.getKey() + "|" + rec.getTitle() + "结果为" + count + "帧里有" + entry.getValue()
                                    + "帧为同一个人");
//                                for (int i = 0; i < faceMap.size(); i++) {
//                                    Log.d("whatTheFuck", entry.getKey() + ":" + entry.getValue());
//                                }
                            Log.d("whatTheFuck", "over!!");
                            if (entry.getValue() >= 3) {
                                if (!rec.getTitle().equals("陌生人") || rec.getDistance() != -1) {
                                    recognizeCount++;
//                                        ft.remove(recognizingFragment);
//                                        frameLayout.setVisibility(View.INVISIBLE);
//                                        if (rec.getDistance() <= 0.4) {
                                    if (rec.getDistance() < 0.8 && rec.getDistance() > 0) {
                                        Intent intent = new Intent();

//                                        intent = new Intent(DetectorActivity.this, ConfirmInfo_recognize.class);
                                        String uuid = UUID.randomUUID().toString();
                                        String facePicUrl = saveSignPicToLocal(uuid);
                                        intent.putExtra("faceFeature", Arrays.deepToString(rec.getEmbeddings()));
                                        intent.putExtra("signInOrOut", signInOrOut);
                                        intent.putExtra("cachePath", uuid);
                                        intent.putExtra("facePicUrl", facePicUrl);
                                        intent.putExtra("id", rec.getTitle());
                                        Log.d("getLocation", "bottom" + rec.getLocation().bottom);
                                        Log.d("getLocation", "top" + rec.getLocation().top);
                                        Log.d("getLocation", "left" + rec.getLocation().left);
                                        Log.d("getLocation", "right" + rec.getLocation().right);
                                        float centerY = (rec.getLocation().bottom - rec.getLocation().top) / 2 + rec.getLocation().top;
                                        float centerX = (rec.getLocation().right - rec.getLocation().left) / 2 + rec.getLocation().left;
                                        Log.d("getLocation", "centerX" + centerX);
                                        Log.d("getLocation", "centerY" + centerY);
                                        intent.putExtra("centerY", centerY);
                                        intent.putExtra("centerX", centerX);
                                        mLog += "\n结果:\n";
                                        for (Map.Entry<String, Integer> entry1 : faceMap.entrySet()) {
                                            mLog = mLog + count + "帧里" + entry1.getKey() + "匹配到" + entry1.getValue()
                                                    + "帧\n";
                                        }
                                        intent.putExtra("signInOrOut", signInOrOut);
                                        Log.d(TAG, "updateResults: 签到还是签退：" + signInOrOut);
                                        intent.putExtra("test", mLog);
                                        Log.d("getLog", "rec.getLOG()" + mLog);
//                                                intent.putExtra("test",rec.getTitle()+":"+rec.getDistance())

                                        Log.d("rec.getTitle()", "rec.getTitle()？" + rec.getTitle());
                                        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                                        faceDetector.close();
//                                        startActivity(intent);
                                        this.setResult(Constants.FACE_REC_RESULT, intent);
                                        finish();
//                                            timer.onFinish();
                                        faceCount.resetFaceCount();
                                        count = 0;
                                    }
                                } else {
                                    Intent intent = new Intent();
//                                    intent = new Intent(DetectorActivity.this, AttendanceFailure.class);
                                    mLog += "\n结果:\n";
                                    for (Map.Entry<String, Integer> entry1 : faceMap.entrySet()) {
                                        mLog = mLog + count + "帧里" + entry1.getKey() + "匹配到" + entry1.getValue()
                                                + "帧\n";
                                    }
                                    intent.putExtra("faceFeature", Arrays.deepToString(rec.getEmbeddings()));
                                    intent.putExtra("test", mLog);
                                    intent.putExtra("signInOrOut", signInOrOut);
                                    Log.d("getLog", "rec.getLOG()" + mLog);
                                    intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                                    faceDetector.close();
//                                    startActivity(intent);
                                    this.setResult(Constants.FACE_REC_RESULT, intent);
                                    faceCount.resetFaceCount();
                                    this.finish();
                                }
                            } else {
                                count2++;
                            }
                            if (count2 != 0) {
                                Intent intent = new Intent();
                                Log.d("recognizeResult", "都不够");
                                mLog += "\n结果:\n";
                                for (Map.Entry<String, Integer> entry1 : faceMap.entrySet()) {
                                    mLog = mLog + count + "帧里" + entry1.getKey() + "匹配到" + entry1.getValue()
                                            + "帧\n";
                                }
                                intent.putExtra("faceFeature", Arrays.deepToString(rec.getEmbeddings()));
                                intent.putExtra("test", mLog);
                                intent.putExtra("signInOrOut", signInOrOut);
                                Log.d("getLog", "rec.getLOG()" + mLog);
//                                intent = new Intent(DetectorActivity.this, AttendanceFailure.class);
                                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                                faceDetector.close();
//                                startActivity(intent);
                                this.setResult(Constants.FACE_REC_RESULT, intent);
                                faceCount.resetFaceCount();
                                this.finish();
                            }
                            faceCount.resetFaceCount();
                            count = 0;
                        }
                    }
                    if (count >= 15) {
                        count = 0;
                        flag = false;
                        faceCount.resetFaceCount();
                    }
//                    }
//                    }
//                }
                }
            }
            isDetected = true;
        }
    }

    //1
    private Matrix drawFrameBitmap() {
        int sourceW = rgbFrameBitmap.getWidth();
        int sourceH = rgbFrameBitmap.getHeight();
        int targetW = portraitBmp.getWidth();
        int targetH = portraitBmp.getHeight();
        //变换矩阵
        Matrix transform = createTransform(
                sourceW,
                sourceH,
                targetW,
                targetH,
                sensorOrientation);
        final Canvas cv = new Canvas(portraitBmp);

        // draws the original image in portrait mode.
        //以纵向模式绘制原始图像
        cv.drawBitmap(rgbFrameBitmap, transform, null);
        return transform;
    }

    //2
    private RectF drawPortraitBitmap(Canvas cvFace, RectF boundingBox, Matrix transform) {
        // maps crop coordinates to original
        //将裁剪坐标映射到原始坐标
        cropToFrameTransform.mapRect(boundingBox);
        // maps original coordinates to portrait coordinates
        //将原始坐标映射到纵向坐标
        RectF faceBB = new RectF(boundingBox);
        transform.mapRect(faceBB);
        // translates portrait to origin and scales to fit input inference size
        //cv.drawRect(faceBB, paint);
        float sx = ((float) TF_OD_API_INPUT_SIZE) / faceBB.width();
        float sy = ((float) TF_OD_API_INPUT_SIZE) / faceBB.height();
        Matrix matrix = new Matrix();
        matrix.postTranslate(-faceBB.left, -faceBB.top);
        matrix.postScale(sx, sy);

        cvFace.drawBitmap(portraitBmp, matrix, null);
        return faceBB;
    }

    //3
    private Bitmap getCrop(RectF faceBB) {
        Bitmap crop = null;
        if ((int) faceBB.left >= 0 &&
                ((int) faceBB.left + (int) faceBB.width()) <= portraitBmp.getWidth() &&
                (int) faceBB.top >= 0 &&
                ((int) faceBB.top + (int) faceBB.height()) <= portraitBmp.getHeight()) {
            crop = Bitmap.createBitmap(portraitBmp,
                    (int) faceBB.left,
                    (int) faceBB.top,
                    (int) faceBB.width(),
                    (int) faceBB.height());
        }
        return crop;
    }

    //4
    //人脸识别

    //5
    private void processRecognizedResults(List<SimilarityClassifier.Recognition> resultsAux) {
        //送进来的图像已经都是居中的了
        //如果有识别结果
        if (resultsAux.size() > 0) {
            //存结果
            SimilarityClassifier.Recognition result = resultsAux.get(0);
            Log.d("result", "result:" + result);
            for (int i = 0; i < resultsAux.size(); i++) {
                Log.d("result", "resultsAux:" + resultsAux.get(i));
            }
            extra = result.getExtra();
            embeddings = result.getEmbeddings();
            mLog = result.getLOG();
            float conf = result.getDistance();
            Log.d("updateResults", "conf:" + conf);
            //如果置信度小于1
            if (conf < 0.8f) {
                confidence = conf;
                label = result.getTitle();
                if (result.getId().equals("0")) {
                    color = Color.GREEN;
                } else {
                    color = Color.RED;
                }
            }
            Log.d("updateResults", "result.getLOG():" + result.getLOG());
            Log.d("updateResults", "rec.getExtra():" + result.getExtra());
            Log.d("updateResults", "confidence:" + result.getDistance());
        }
    }

    //6
    private void flipImage(RectF boundingBox) {
        if (getCameraFacing() == CameraCharacteristics.LENS_FACING_FRONT) {

            // camera is frontal so the image is flipped horizontally
            //相机位于正面，因此图像水平翻转
            // flips horizontally
            Matrix flip = new Matrix();
            if (sensorOrientation == 90 || sensorOrientation == 270) {
                flip.postScale(1, -1, previewWidth / 2.0f, previewHeight / 2.0f);
            } else {
                flip.postScale(-1, 1, previewWidth / 2.0f, previewHeight / 2.0f);
            }
            //flip.postScale(1, -1, targetW / 2.0f, targetH / 2.0f);
            flip.mapRect(boundingBox);

        }
    }
    //获得5张人脸->
    //处理5张符合标准的人脸

    private List<SimilarityClassifier.Recognition> createMappedRecognitions(RectF faceBB, Bitmap crop, List<SimilarityClassifier.Recognition> mappedRecognitions) {
        final SimilarityClassifier.Recognition result = new SimilarityClassifier.Recognition(
                "0", label, confidence, faceBB);
        Log.d("updateResults", "-------result.getDistance():" + result.getDistance() + "confidence" + confidence);

        result.setColor(color);
        result.setLocation(faceBB);
        result.setExtra(extra);
        result.setEmbeddings(embeddings);
        //裁剪的人脸会放到updateResults里的showAddFaceDialog
        result.setCrop(crop);
        mappedRecognitions.add(result);
        Log.d("mappedRecognitions", "result=" + mappedRecognitions.get(0).getTitle());
        Log.d("cao", "result=" + result);
        Log.d("updateResults", "result.getExtra():" + result.getExtra());

        Log.d("mappedRecognitions==", "mappedRecognitions.size()=" + mappedRecognitions.size());

        return mappedRecognitions;
    }

    private void onFacesDetected(List<Face> faces) {
        final Paint paint = new Paint();
        paint.setColor(Color.RED);
        paint.setStyle(Style.STROKE);
        paint.setStrokeWidth(2.0f);
//---------------------------------------------------
        // Note this can be done only once
        //注意这只能做一次
        Matrix transform = drawFrameBitmap();
        final Canvas cvFace = new Canvas(faceBmp);
        Log.d("face", "faces有几个+" + faces.size());
        int max = 0;

//        final List<SimilarityClassifier.Recognition> resultsAux = detector.recognizeImage(faceBmp, registerOrRecognize);
//        Log.d("resultsAux","resultsAux---liu:"+resultsAux.get(0).getDistance());
        if (faces.size() == 1) {

        }
        for (Face face : faces) {
            FaceResults faceResults = new FaceResults();
            //获取人脸框
            final RectF boundingBox = new RectF(face.getBoundingBox());
            final boolean goodConfidence = true; //face.get;
            int temp;
            //如果人脸框不为空
            if (boundingBox != null && goodConfidence) {
                RectF faceBB = drawPortraitBitmap(cvFace, boundingBox, transform);//faceBB用来判断是否居中
                Bitmap crop = getCrop(faceBB);//crop用来计算人脸框面积
                final long startTime = SystemClock.uptimeMillis();
                Log.d("isDetected", "isDetected:" + isDetected + ",(int) faceBB.width() < portraitBmp.getWidth() *2/3:" + ((int) faceBB.width() < portraitBmp.getWidth() * 2 / 3));
                //人脸识别
                //add就是判断是否需要注册人脸的标识
                //resultsAux存放了识别的人脸信息
                //这里需要人脸尽量居中对齐，所以加了判断矩形框坐标相对整个屏幕的比例
                if ((
                        (int) faceBB.left > portraitBmp.getWidth() / 10
                                && (int) faceBB.right < portraitBmp.getWidth() * 9 / 10
                                && (int) faceBB.top > portraitBmp.getHeight() / 10
                                && (int) faceBB.bottom < portraitBmp.getHeight() * 7 / 10
//                        && (int) faceBB.width() > portraitBmp.getWidth() / 4
                )
                        && crop != null
                ) {
                    //如果居中满足要求,就返回一个标识,保存这一帧
                    //待修改.....
                    //这里加一个是否居中的标识,和一个注册或识别标识
                    temp = crop.getHeight() * crop.getWidth();
                    if (temp > max) {
                        isCentered = true;
                        Bitmap recFace = faceBmp.copy(Config.ARGB_8888, true);//把当前bitmap赋值给待滤镜处理的bitmap
                        //保存了这一帧的重要信息
                        Log.d("onFacesDetected", "faces有几个:" + faces.size());
                        faceResults.setFaceBB(faceBB);
                        Log.d("onFacesDetected", "faceBmp:" + recFace.getPixel(50, 50));
                        faceResults.setFaceBmp(recFace);
//                        Log.d("onFacesDetected","crop:"+crop);
                        faceResults.setCrop(crop);
                        faceResults.setBoundingBox(boundingBox);
                        listFaceResults.add(faceResults);
                        Log.d("onFacesDetected", "faceResults" + faceResults.getFaceBmp().getPixel(50, 50));
                    }
//                    for (int i = 0;i<listFaceResults.size();i++){
//                        Log.d("loop",i+":bmp:"+listFaceResults.get(i).getFaceBmp().getPixel(50,50));
//                        Log.d("loop",i+":crop:"+listFaceResults.get(i).getCrop().getPixel(50,50));
//                        Log.d("loop",i+":faceBB:"+listFaceResults.get(i).getFaceBB());
//                    }
                    if (listFaceResults.size() == 15) {
                        isDetected = false;
                    }
                } else {
                    isCentered = false;
                }

            }
        }
    }


    //主入口


    private List<SimilarityClassifier.Recognition> process5face() {
        RectF faceBB;
        Bitmap crop;
        Bitmap faceBmp;
        RectF boundingBox;
        List<SimilarityClassifier.Recognition> mappedRecognitions = new LinkedList<>();
        Log.d("listFaceResults", "listFaceResults.size:" + listFaceResults.size());
        if (listFaceResults.size() == 15) {
            int count = 0;
            for (int i = 10; i < listFaceResults.size(); i++) {
                Log.d("process5face", "listFaceResults(" + i + ")" + listFaceResults.get(i).getFaceBmp().getPixel(50, 50));
                faceBB = listFaceResults.get(i).getFaceBB();
                crop = listFaceResults.get(i).getCrop();
                boundingBox = listFaceResults.get(i).getBoundingBox();
                faceBmp = listFaceResults.get(i).getFaceBmp();
                recFlag = true;
//                }

                Log.d("process5face", "计数：" + count++);
                Log.d("process5face", "crop：" + crop.getWidth());
                Log.d("process5face", "faceBmp：" + faceBmp.getWidth());
                Log.d("process5face", "faceBmp:" + faceBmp.getPixel(50, 50));
                Log.e("recognizeImage", "---------第" + (i - 9) + "帧比对--------");
                final List<SimilarityClassifier.Recognition> resultsAux = detector.recognizeImage(faceBmp, "recognize", i - 9);
                processRecognizedResults(resultsAux);
                flipImage(boundingBox);
                mappedRecognitions = createMappedRecognitions(faceBB, crop, mappedRecognitions);
            }
        }
        return mappedRecognitions;
    }

    //ondetect:得到crop,faceBB,faceBmp,做完之后不需要再循环face,而是循环保存的20张faceBmp的最后5帧
    //本身不需要输出什么
    //绘制工作在updateResults中,所以不需要mappedRecognitions


}
