package quake.quake_tensorflow_face_rec.ui.CustomView;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.View;

import androidx.annotation.Nullable;

import quake.quake_tensorflow_face_rec.R;


public class SignButton extends View {
    private Paint roundPaint;
    private Paint textPaint;
    private int roundWidth;
    private int roundColor;
    private String text;
    private int textColor;
    private int textSize;
    private int x_radius;
    private int y_radius;
    private boolean inOrOut;

    public SignButton(Context context) {
        this(context,null);
    }

    public SignButton(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public SignButton(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.SignButton,defStyleAttr,0);
        int n = typedArray.getIndexCount();
        for (int i = 0;i<n;i++){
            int attr = typedArray.getIndex(i);
            if (attr == R.styleable.SignButton_roundWidth) {
                roundWidth = typedArray.getDimensionPixelSize(attr, (int) TypedValue.applyDimension(
                        TypedValue.COMPLEX_UNIT_PX, 5, getResources().getDisplayMetrics()));
            } else if (attr == R.styleable.SignButton_roundColor) {
                roundColor = typedArray.getColor(attr, Color.WHITE);
            } else if (attr == R.styleable.SignButton_text) {
                text = typedArray.getString(attr);
            } else if (attr == R.styleable.SignButton_textColor) {
                textColor = typedArray.getColor(attr, Color.BLACK);
            } else if (attr == R.styleable.SignButton_textSize) {
                textSize = typedArray.getDimensionPixelSize(attr, (int) TypedValue.applyDimension(
                        TypedValue.COMPLEX_UNIT_PX, 5, getResources().getDisplayMetrics()));
            } else if (attr == R.styleable.SignButton_x_radius) {
                x_radius = typedArray.getDimensionPixelSize(attr, (int) TypedValue.applyDimension(
                        TypedValue.COMPLEX_UNIT_PX, 5, getResources().getDisplayMetrics()));
            } else if (attr == R.styleable.SignButton_y_radius) {
                y_radius = typedArray.getDimensionPixelSize(attr, (int) TypedValue.applyDimension(
                        TypedValue.COMPLEX_UNIT_PX, 5, getResources().getDisplayMetrics()));
            } else if (attr == R.styleable.SignButton_inOrOut) {
                inOrOut = typedArray.getBoolean(attr, false);
            }
        }
        typedArray.recycle();
        roundPaint = new Paint();
        textPaint = new Paint();
    }

    public int getRoundColor() {
        return roundColor;
    }

    public void setRoundColor(int roundColor) {
        this.roundColor = roundColor;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
//        mPaint.setTypeface()
        int radius = getHeight();
        roundPaint.setStyle(Paint.Style.FILL);
        roundPaint.setColor(roundColor);
        roundPaint.setAntiAlias(true);
        if(inOrOut){
            canvas.drawCircle(x_radius, y_radius, (float) (radius*5/6), roundPaint);
        }else {
            canvas.drawCircle(getHeight()+x_radius, getHeight()+y_radius, (float) (radius*5/6), roundPaint);

        }

        int centreX;
        int centreY;
        if(inOrOut){
            centreX = getWidth() *3/ 7;
            centreY = getHeight() *3/ 7;
            textPaint.setTextAlign(Paint.Align.CENTER);
        }else {
            centreX = getWidth()*5/ 7;
            centreY = getHeight()*5/7;
            textPaint.setTextAlign(Paint.Align.LEFT);
        }
        textPaint.setColor(textColor);
        textPaint.setTextSize(textSize);

        Paint.FontMetrics fontMetrics = textPaint.getFontMetrics();

        canvas.drawText(text,(float) (centreY),centreY,textPaint);
        postInvalidate();
    }
}
