/**
 * Copyright (C), 2021-2021, 夸克科技有限公司
 * FileName: JOUtils
 * Author: Administrator
 * Date: 2021/10/15 15:02
 * Description: JSONObject工具类
 * History:
 * <author> <time> <version> <desc>
 * 作者姓名 修改时间 版本号 描述
 */
package quake.quake_tensorflow_face_rec.utils.JSONUtils;

import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;

/**
 * @ClassName: JOUtils
 * @Description: java类作用描述
 * @Author: Administrator
 * @Date: 2021/10/15 15:02
 */
public class JsonUtils {
    /**
     * 插入数据到HashMap
     *
     * @param container HashMap容器
     * @param args      键值对交替写入，偶数key，奇数value
     * @return 返回一个HashMap
     */
    public static HashMap<Object, Object> inputValue2HashMap(HashMap<Object, Object> container, Object... args) {
        for (int i = 0; i < args.length; i++) {
            if (i % 2 == 0) {
                container.put(args[i], args[i + 1]);
            }
        }
        return container;
    }

    /**
     * 插入数据到JSONObject
     *
     * @param container JSONObject容器
     * @param args      键值对交替写入，偶数key，奇数value
     * @return 返回一个JSONObject
     */
    public static JSONObject inputValue2JSONObject(JSONObject container, Object... args) {
        for (int i = 0; i < args.length; i++) {
            if (i % 2 == 0) {
                try {
                    container.put((String) args[i], args[i + 1]);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        }
        return container;
    }

    /**
     * 插入数据到JSONArray
     *
     * @param container JSONArray容器
     * @param args      数据
     * @return JSONArray
     */
    public static JSONArray inputValue2JSONArray(JSONArray container, Object... args) {
        for (int i = 0; i < args.length; i++) {
            container.put(args[i]);
        }
        return container;
    }


    /**
     * JSONArray转JSONObject
     *
     * @param jsonArray jsonArray
     * @return jsonObject
     * @throws JSONException
     */
    public static JSONObject jsonArray2jsonObject(JSONArray jsonArray) throws JSONException {
        JSONObject js = new JSONObject();
        for (int i = 0; i < jsonArray.length(); i++) {
            JSONObject ob = (JSONObject) jsonArray.get(i);
            Log.d("jsonArray2jsonObject", "ob:" + ob.toString());
            Iterator itt = ob.keys();

            while (itt.hasNext()) {
                String key = itt.next().toString();
                Log.d("jsonArray2jsonObject", "key:" + key);
                String value = ob.getString(key);
                js.put(key, value);
            }
        }
        return js;
    }

    /**
     * 将JSONObject格式的数据，根据key提取value
     *
     * @param response 响应数据
     * @param myKey    键
     * @return 值
     */
    public static String getValue(JSONObject response, String myKey) {
        try {
            String s = response.toString();
            JSONObject jsonObject = new JSONObject(s);
            //层层解析
            Iterator itt = jsonObject.keys();
            while (itt.hasNext()) {
                String key = itt.next().toString();
                String value = jsonObject.getString(key);
                if (key.equals(myKey)) {
                    return value;
                }
                /*if (value==null){
                    continue;
                }else if("".equals(value.trim())){
                    resObj.put(key,value.trim());
                }*/
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
        /*try {
            String s = response.toString();
            JSONObject jsonObject = new JSONObject(s);
            String msg =jsonObject.getString(key);
            Log.d("myApp",
                    "-----------------------------------------取得value值：" + msg);
            return msg;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;*/
    }
    /**
     * 将jsonString形式的数据，根据所提供的key，取出value
     * @param res 响应得到的jsonString数据
     * @return
     */
    public static String getValue(String res, String myKey){
        try{
            //将Json字符串转换成JSONObject格式
            JSONObject resObj = new JSONObject(res);
            //层层解析
            Iterator itt = resObj.keys();
            while (itt.hasNext()){
                String key = itt.next().toString();
                String value = resObj.getString(key);
                if (key.equals(myKey)){
                    return value;
                }
                /*if (value==null){
                    continue;
                }else if("".equals(value.trim())){
                    resObj.put(key,value.trim());
                }*/
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
    }


    private static final String TAG = "jsonUtils";


    /**
     * 封装String格式数据为json数据
     * @param key
     * @param value
     * @return
     */
    public static JSONObject string2JSONObject(JSONObject myJson, String key, String value) {
        JSONObject json = myJson;
        try {
            json.put(key,value);

            Log.d(TAG, "json: "+json);
            return json;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return json;
    }

    public static JSONArray string2JSONArray(String string){
        try {
            JSONArray jsonArray = new JSONArray(string);
            return jsonArray;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
    }

}
