/*
 * Copyright 2016 Tzutalin
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package quake.quake_tensorflow_face_rec.utils;

import android.content.Context;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.RawRes;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Created by darrenl on 2016/3/30.
 * Modified by Gaurav on Feb 23, 2018
 */
public class FileUtils {

    public static int getFileSizeByUri(Context context,Uri uri){
        InputStream input = null;
        int size = 0;
        try {
            input =context. getContentResolver().openInputStream(uri);
            size = input.available();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return size;
    }
    public static String getFileName(String pathandname) {
        /**
         * 仅保留文件名不保留后缀
         */
        int start = pathandname.lastIndexOf("/");
        int end = pathandname.lastIndexOf(".");
        if (start != -1 && end != -1) {
            return pathandname.substring(start + 1, end);
        } else {
            return null;
        }
    }

    /**
     * 保留文件名及后缀
     */
    public static String getFileNameWithSuffix(String pathandname) {
        int start = pathandname.lastIndexOf("/");
        if (start != -1) {
            return pathandname.substring(start + 1);
        } else {
            return null;
        }
    }

    /**
     * 根据文件路径获取文件后缀
     * @param path 文件路径
     * @return 后缀名
     */
    public static String parseFileType(String path) {
        if (TextUtils.isEmpty(path)) {
            return "";
        } else {
            return path.substring(path.lastIndexOf(".") + 1);
        }
    }

    /**
     * 获取文件类型
     * @param paramString
     * @return
     */
    public static String getFileType(String paramString) {
        String str = "";
        if (TextUtils.isEmpty(paramString)) {
            Log.d("print", "paramString---->null");
            return str;
        }
        Log.d("print", "paramString:" + paramString);
        int i = paramString.lastIndexOf('.');
        if (i <= -1) {
            Log.d("print", "i <= -1");
            return str;
        }

        str = paramString.substring(i + 1);
        Log.d("print", "paramString.substring(i + 1)------>" + str);
        return str;
    }


    @NonNull
    public static final void copyFileFromRawToOthers(@NonNull final Context context, @RawRes int id, @NonNull final String targetPath) {
        InputStream in = context.getResources().openRawResource(id);
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(targetPath);
            byte[] buff = new byte[1024];
            int read = 0;
            while ((read = in.read(buff)) > 0) {
                out.write(buff, 0, read);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void copyFile(String srcPath, String targetPath) {
        InputStream in = null;
        OutputStream out = null;
        try {
            in = new FileInputStream(srcPath);
            out = new FileOutputStream(targetPath);
            byte[] buf = new byte[1024];
            int len;
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 判断文件是否存在
     *
     * @param strFile 文件名+后缀
     * @return 布尔值
     */
    public static boolean fileIsExists(Context context, String strFile) {
        String string = null, s = null;
        Log.d("version", "version:" + Build.VERSION.SDK_INT);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            string = context.getFilesDir().getAbsolutePath(); //此路径便是文件存储在应用程序中的默认路径
            //getFilesDir().getCanonicalPath得到的是规范路径，与我上面描述的默认路径一致
            //getFilesDir().getPath()得到的路径就是你写的什么，就得到什么
            Log.w("TAG", string + "\n" + "\n" + context.getFilesDir().getPath() + "\n\n"
//                    + context.getDataDir().getAbsolutePath() + "\n" + "\n" + context.getDataDir().getPath()
            );
//            s = context.getDataDir().getAbsolutePath() + "/shared_prefs/" + strFile;
//            s = context.getDir("shared_prefs",Context.MODE_PRIVATE).getAbsolutePath()+ strFile;
            s = Environment.getDataDirectory().getAbsolutePath() + "/data/org.tensorflow.lite.examples.facerecognition/shared_prefs/" + strFile;
        }
        Log.d("fileIsExists", "s:" + s);
        File file = new File(s);
        Log.d("TAG:LOGIN_ACTIVITY", "" + file.exists());
        return file.exists();


    }

    public int CopyFile(String fromFile, String toFile) {

        try {
            InputStream fosfrom = new FileInputStream(fromFile);
            OutputStream fosto = new FileOutputStream(toFile);
            byte bt[] = new byte[1024];
            int c;
            while ((c = fosfrom.read(bt)) > 0) {
                fosto.write(bt, 0, c);
            }
            fosfrom.close();
            fosto.close();
            return 0;

        } catch (Exception ex) {
            return -1;
        }
    }

    //获取缓存地址
    public static String getDiskCacheDir(Context context) {
        String cachePath = null;

        if (Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState())
                || !Environment.isExternalStorageRemovable()) {
            //SD卡存在
            cachePath = context.getExternalCacheDir().getPath();

        } else {
            cachePath = context.getCacheDir().getPath();
        }
        return cachePath;

    }

}
